# YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
PyTorch Hub yolo_models https://pytorch.org/hub/ultralytics_yolov5/
Usage:
	import torch
	model = torch.hub.load('ultralytics/models', 'yolov5s')
"""

import torch

def _create(name, pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	"""Creates a specified YOLOv5 model
	Arguments:
		name (str): name of model, i.e. 'yolov5s'
		pretrained (bool): load pretrained weights into the model
		channels (int): number of input channels
		classes (int): number of model classes
		autoshape (bool): apply YOLOv5 .autoshape() wrapper to model
		verbose (bool): print all information to screen
		device (str, torch.device, None): device to use for model parameters
	Returns:
		YOLOv5 pytorch model
	"""
	from pathlib import Path
	import os
	from soco_yolo.yolo import Model
	from soco_yolo.experimental import attempt_load
	from soco_yolo.utils.general import check_requirements, set_logging
	from soco_yolo.utils.downloads import attempt_download
	from soco_yolo.utils.torch_utils import select_device
	from soco_yolo.config import EnvVar

	check_requirements(exclude=('tensorboard', 'thop', 'opencv-python'))
	set_logging(verbose=verbose)

	# save_dir = Path('') if str(name).endswith('.pt') else file.parent
	root_dir = os.path.dirname(os.path.abspath(__file__)).replace('soco_yolo', EnvVar.MODEL_DIR)
	save_dir = Path(root_dir).resolve()
	path = (save_dir / name).with_suffix('.pt')  # checkpoint path
	try:
		device = EnvVar.DEVICE
		if pretrained and channels == 3 and classes == 80:
			#model = attempt_load(path, map_location=device)  # download/load FP32 model
			model = attempt_load(name, map_location=device)  # download/load FP32 model
		else:
			cfg = list((Path(__file__).parent / 'yolo_models').rglob(f'{name}.yaml'))[0]  # model.yaml path
			model = Model(cfg, channels, classes)  # create model
			if pretrained:
				ckpt = torch.load(attempt_download(path), map_location=device)	# load
				msd = model.state_dict()  # model state_dict
				csd = ckpt['model'].float().state_dict()  # checkpoint state_dict as FP32
				csd = {k: v for k, v in csd.items() if msd[k].shape == v.shape}  # filter
				model.load_state_dict(csd, strict=False)  # load
				if len(ckpt['model'].names) == classes:
					model.names = ckpt['model'].names  # set class names attribute
		if autoshape:
			model = model.autoshape()  # for file/URI/PIL/cv2/np inputs and NMS

		return model.to(device)

	except Exception as e:
		help_url = 'https://github.com/ultralytics/yolov5/issues/36'
		s = 'Cache may be out of date, try `force_reload=True`. See %s for help.' % help_url
		raise Exception(s) from e


def custom(path='path/to/model.pt', autoshape=True, verbose=True, device=None):
	# YOLOv5 custom or local model
	return _create(path, autoshape=autoshape, verbose=verbose, device=device)


def yolov5s(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-small model https://github.com/ultralytics/yolov5
	return _create('yolov5s', pretrained, channels, classes, autoshape, verbose, device)


def yolov5m(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-medium model https://github.com/ultralytics/yolov5
	return _create('yolov5m', pretrained, channels, classes, autoshape, verbose, device)


def yolov5l(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-large model https://github.com/ultralytics/yolov5
	return _create('yolov5l', pretrained, channels, classes, autoshape, verbose, device)


def yolov5x(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-xlarge model https://github.com/ultralytics/yolov5
	return _create('yolov5x', pretrained, channels, classes, autoshape, verbose, device)


def yolov5s6(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-small-P6 model https://github.com/ultralytics/yolov5
	return _create('yolov5s6', pretrained, channels, classes, autoshape, verbose, device)


def yolov5m6(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-medium-P6 model https://github.com/ultralytics/yolov5
	return _create('yolov5m6', pretrained, channels, classes, autoshape, verbose, device)


def yolov5l6(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-large-P6 model https://github.com/ultralytics/yolov5
	return _create('yolov5l6', pretrained, channels, classes, autoshape, verbose, device)


def yolov5x6(pretrained=True, channels=3, classes=80, autoshape=True, verbose=True, device=None):
	# YOLOv5-xlarge-P6 model https://github.com/ultralytics/yolov5
	return _create('yolov5x6', pretrained, channels, classes, autoshape, verbose, device)


if __name__ == '__main__':
	model = _create(name='yolov5s', pretrained=True, channels=3, classes=80, autoshape=True, verbose=True)	# pretrained
	# model = custom(path='path/to/model.pt')  # custom

	# Verify inference
	import cv2
	import numpy as np
	from PIL import Image
	from pathlib import Path

	imgs = ['data/images/zidane.jpg',  # filename
			Path('data/images/zidane.jpg'),  # Path
			'https://ultralytics.com/images/zidane.jpg',  # URI
			cv2.imread('data/images/bus.jpg')[:, :, ::-1],	# OpenCV
			Image.open('data/images/bus.jpg'),	# PIL
			np.zeros((320, 640, 3))]  # numpy

	results = model(imgs)  # batched inference
	results.print()
	results.save()



from fastapi import APIRouter

from models.http.routes import heartbeat, detector

router = APIRouter()
router.include_router(heartbeat.router, tags=["health"], prefix="/v1/health")
router.include_router(detector.router, tags=["detect"], prefix="/v1/serving")





import os
import yaml
import json

class EnvVar(object):
	APP_VERSION = "1.0"
	APP_NAME = "Yolov5"
	root_dir = os.path.dirname(os.path.realpath(__file__)).replace('soco_yolo', '')

	#default = yaml.load(open(os.path.join(root_dir, 'configs/default.yaml'), 'r'), Loader=yaml.FullLoader)

	#API_PREFIX = os.environ.get('API_PREFIX', default['API_PREFIX'])
	API_PREFIX = "/yolo"
	#IS_DEBUG = bool(os.environ.get('IS_DEBUG', default['IS_DEBUG']))
	IS_DEBUG = False
	#MODEL_DIR = os.environ.get("MODEL_DIR", default['MODEL_DIR'])
	MODEL_DIR = "./"
	#DEVICE = os.environ.get("DEVICE", default['DEVICE'])
	DEVICE = "cuda"
	#BATCH_SIZE = int(os.environ.get("BATCH_SIZE", default['BATCH_SIZE']))
	BATCH_SIZE = 160


	@classmethod
	def deepcopy(cls, x):
		return json.loads(json.dumps(x))







from fastapi import FastAPI
from models.http.routes.router import router
from models.config import EnvVar
from models.http.event_handlers import (start_app_handler, stop_app_handler, exception_handler)


def get_app() -> FastAPI:
    fast_app = FastAPI(title=EnvVar.APP_NAME, version=EnvVar.APP_VERSION, debug=EnvVar.IS_DEBUG)
    fast_app.include_router(router, prefix=EnvVar.API_PREFIX)

    fast_app.add_event_handler("startup", start_app_handler(fast_app))
    fast_app.add_event_handler("shutdown", stop_app_handler(fast_app))
    fast_app.add_exception_handler(Exception, exception_handler)

    return fast_app


app = get_app()




from soco_yolo.bases import ObjectDetector
import os
import torch
import urllib
import numpy as np
from PIL import Image
from typing import List, Union
from soco_yolo.utils.helpers import LRUCache, chunks
from soco_yolo.config import EnvVar
from soco_yolo.hubconf import custom
import requests
import io
import base64
from soco_yolo.hubconf import custom


class Yolov5(ObjectDetector):
	root_dir = os.path.dirname(os.path.abspath(__file__))
	MAX_CACHE = 20

	def __init__(self):
		self._models = LRUCache(self.MAX_CACHE)
		self.batch_size = EnvVar.BATCH_SIZE

	def _load_model(self, model_id):
		if model_id is None:
			raise Exception("Model ID cannot be None.")

		if not self._models.has(model_id):
			# model = torch.hub.load(self.root_dir, model_id, source='local', pretrained=True)
			model = custom(model_id)
			model = model.to(EnvVar.DEVICE)
			model.eval()
			self._models.put(model_id, model)

		return self._models.get(model_id)

	def predict(self, model_id, data: List,
				src_type: str = 'local',
				threshold: float = 0.5,
				include_classes: Union[List, None] = None):
		if src_type == 'local':
			image_data = [Image.open(x).convert('RGB') for x in data]

		elif src_type == 'url':
			image_data = []
			for x in data:
				# temp = Image.open(io.BytesIO(requests.get(x).content))
				# temp = Image.open(requests.get(x, stream=True).raw).convert('RGB')
				temp = Image.open(urllib.request.urlopen(x)).convert('RGB')
				image_data.append(temp)

		elif src_type == 'base64':
			image_data = []
			for x in data:
				temp = Image.open(io.BytesIO(base64.b64decode(x)))
				image_data.append(temp)
		else:
			raise Exception("Unknown mode {}.".format(src_type))

		if include_classes is not None:
			include_classes = set(include_classes)
		
		model = self._load_model(model_id)

		resp = []
		for batch in chunks(image_data, self.batch_size):
			with torch.no_grad():
				y = model(batch)

			for z in y.tolist():
				temp = []
				for pred in z.pred:
					x, y, xx, yy, conf, cls = pred
					cls = z.files[int(cls)]
					conf = float(conf)
					if conf > threshold:
						if include_classes is None or cls in include_classes:
							temp.append({'xmin': float(x),
										 'ymin': float(y),
										 'xmax': float(xx),
										 'ymax': float(yy),
										 'conf': conf,
										 'label': cls})

				resp.append(temp)

		return resp

	def destroy_model(self):
		self._models = LRUCache(self.MAX_CACHE)




import setuptools
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="soco_yolo",
    packages = find_packages(),
    package_data={'': ['*.yaml']},
    include_package_data=True,
    version="0.0.5",
    author="kyusonglee",
    description="YOLO wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.soco.ai",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Free for non-commercial use",
        "Operating System :: OS Independent",
    ],
    install_requires = [
        "Pillow==8.2.0",
        "tqdm==4.56.0",
        "requests >= 2.25.0",
        "matplotlib>=3.2.2",
        "numpy>=1.18.5",
        "opencv-python>=4.1.2",
        "PyYAML>=5.3.1",
        "scipy>=1.4.1",
        "torch==1.8.1",
        "torchvision==0.9.1",
        "pandas",
        "seaborn>=0.11.0",
        "thop"
    ]
)



from fastapi import APIRouter

from models.http.schemas.heartbeat import Heartbeat

router = APIRouter()


@router.get("/ping", response_model=Heartbeat, name="ping")
def ping() -> Heartbeat:
    heartbeat = Heartbeat(is_alive=True)
    return heartbeat



from typing import Callable
from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse
from models.engine import Yolov5

import logging
logger = logging.getLogger(__name__)


def start_app_handler(app: FastAPI) -> Callable:
    def startup() -> None:
        logger.info("Running app start handler.")
        app.state.detector = Yolov5()

    return startup


def stop_app_handler(app: FastAPI) -> Callable:
    def shutdown() -> None:
        logger.info("Running app shutdown handler.")
        app.state.detector = None

    return shutdown


async def exception_handler(request: Request, exc: Exception):
    return JSONResponse(
        status_code=500,
        content={'error': str(exc)},
    )


from models.bases import ObjectDetector
import os
import torch
import urllib
import numpy as np
from PIL import Image
from typing import List, Union
from models.utils.helpers import LRUCache, chunks
from models.config import EnvVar
from models.hubconf import custom
import requests
import io
import base64
from models.hubconf import custom


class Yolov5(ObjectDetector):
	root_dir = os.path.dirname(os.path.abspath(__file__))
	MAX_CACHE = 20

	def __init__(self):
		self._models = LRUCache(self.MAX_CACHE)
		self.batch_size = EnvVar.BATCH_SIZE

	def _load_model(self, model_id):
		if model_id is None:
			raise Exception("Model ID cannot be None.")

		if not self._models.has(model_id):
			# model = torch.hub.load(self.root_dir, model_id, source='local', pretrained=True)
			model = custom(model_id)
			model = model.to(EnvVar.DEVICE)
			model.eval()
			self._models.put(model_id, model)

		return self._models.get(model_id)

	def predict(self, model_id, data: List,
				src_type: str = 'local',
				threshold: float = 0.5,
				include_classes: Union[List, None] = None):
		if src_type == 'local':
			image_data = [Image.open(x).convert('RGB') for x in data]

		elif src_type == 'url':
			image_data = []
			for x in data:
				# temp = Image.open(io.BytesIO(requests.get(x).content))
				# temp = Image.open(requests.get(x, stream=True).raw).convert('RGB')
				temp = Image.open(urllib.request.urlopen(x)).convert('RGB')
				image_data.append(temp)

		elif src_type == 'base64':
			image_data = []
			for x in data:
				temp = Image.open(io.BytesIO(base64.b64decode(x)))
				image_data.append(temp)
		else:
			raise Exception("Unknown mode {}.".format(src_type))

		if include_classes is not None:
			include_classes = set(include_classes)
		
		model = self._load_model(model_id)

		resp = []
		for batch in chunks(image_data, self.batch_size):
			with torch.no_grad():
				y = model(batch)

			for z in y.tolist():
				temp = []
				for pred in z.pred:
					x, y, xx, yy, conf, cls = pred
					cls = z.files[int(cls)]
					conf = float(conf)
					if conf > threshold:
						if include_classes is None or cls in include_classes:
							temp.append({'xmin': float(x),
										 'ymin': float(y),
										 'xmax': float(xx),
										 'ymax': float(yy),
										 'conf': conf,
										 'label': cls})

				resp.append(temp)

		return resp

	def destroy_model(self):
		self._models = LRUCache(self.MAX_CACHE)

